this.EXPORTED_SYMBOLS = ["CustomizableUI"];


const wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);

var gLoadedWidgets = new Set();


this.CustomizableUI = {

	shim: true,
	
	createWidget: function (aProperties) {
		const newWidgetId = aProperties.id;
		for (let aProperties of gLoadedWidgets) {
			if(aProperties.id == newWidgetId) {
				return;
			}
		}
		
		gLoadedWidgets.add(aProperties);
		
		const windows = wm.getEnumerator(null);
		while ( windows.hasMoreElements() ) {
			addWidgetToWindow( windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindow) , aProperties );
		}
		
		wm.addListener(windowListener);
	},
	
	destroyWidget: function (aWidgetId) {
		const windows = wm.getEnumerator(null);
		var doc, button, toolboxes, toolboxesLen, toolbox, buttons, buttonsLen;
		while ( windows.hasMoreElements() ) {
			doc = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindow).document;
			button = doc.getElementById(aWidgetId);
			if (button) {
				button.parentNode.removeChild(button);
			} else {
				toolboxes = doc.getElementsByTagName("toolbox");
				
				toolboxesLen = toolboxes.length;
				for(let i = 0; i < toolboxesLen; i++) {
					toolbox = toolboxes[i];
					if(!toolbox.palette) {
						continue;
					}
					buttons = toolbox.palette.getElementsByAttribute("id", aWidgetId);
					
					buttonsLen = buttons.length;
					for (let j = 0; j < buttonsLen; j++) {
						try{
							toolbox.palette.removeChild(buttons[j]);
						}catch(e){
							
						}
					}
				}
			}
		}
		
		var widgetCount = 0;
		for (let aProperties of gLoadedWidgets) {
			if(aProperties.id) {
				if(aProperties.id == aWidgetId) {
					gLoadedWidgets.delete(aProperties);
				} else {
					widgetCount++;
				}
			}
		}
		if(!widgetCount) {
			wm.removeListener(windowListener);
		}
	}

}


function addWidgetToWindow(window, aProperties) {
	const doc = window.document;
	
	const toolboxId = aProperties.toolbox ? aProperties.toolbox : "navigator-toolbox";
	const toolbox = doc.getElementById(toolboxId);
	if(!toolbox) {
		return;
	}
	const button = doc.createElement("toolbarbutton");
	if (aProperties.id) {
		button.setAttribute("id", aProperties.id);
	} else {
		return;
	}
	aProperties.label ? button.setAttribute("label", aProperties.label) : null;
	aProperties.tooltiptext ? button.setAttribute("tooltiptext", aProperties.tooltiptext) : null;
	aProperties.disabled ? button.setAttribute("disabled", aProperties.disabled) : null;
	aProperties.stop ? button.setAttribute("stop", aProperties.stop) : null;
	aProperties.onCommand ? button.addEventListener("command", aProperties.onCommand, false) : null;
	aProperties.onClick ? button.addEventListener("click", aProperties.onClick, false) : null;
	button.classList.add("toolbarbutton-1");
	button.classList.add("chromeclass-toolbar-additional");
	
	toolbox.palette.appendChild(button);
	
	restoreToToolbar(window, aProperties);
}

//Close window of chrome://global/content/customizeToolbar.xul in function shutdown() in bootstrap.js .
function restoreToToolbar(window, aProperties) {
	const doc = window.document;
	
	const toolboxId = aProperties.toolbox ? aProperties.toolbox : "navigator-toolbox";
	const toolbox = doc.getElementById(toolboxId);
	if(!toolbox) {
		return;
	}
	const aWidgetId = aProperties.id
	
	var potentialToolbars = Array.slice(toolbox.getElementsByTagName("toolbar"));
	if(toolbox.externalToolbars) {
		potentialToolbars = potentialToolbars.concat(toolbox.externalToolbars);
	}
	var toolbar, buttonSet, buttons, pos, spacers, beforeNode, buttonsLen, nodeId;
	
	const poTbLen = potentialToolbars.length;
	for(let i = 0; i < poTbLen; i++) {
		toolbar = potentialToolbars[i];
		buttonSet = toolbar.getAttribute("currentset");
		buttons = buttonSet.split(",");
		pos = buttons.indexOf(aWidgetId);
		if(pos != -1) {
			spacers = 0;
			beforeNode = toolbar.ownerDocument.getElementById(buttons[pos]);
			beforeNode = (beforeNode != null && beforeNode.parentNode != toolbar) ? null : beforeNode;
			
			buttonsLen = buttons.length;
			while(beforeNode == null && pos < buttonsLen) {
				nodeId = buttons[pos];
				if(nodeId == "spacer" || nodeId == "separator" || nodeId == "spring") {
					spacers++;
				} else {
					beforeNode = toolbar.ownerDocument.getElementById(nodeId);
					beforeNode = (beforeNode != null && beforeNode.parentNode != toolbar) ? null : beforeNode;
				}
				pos++;
			}
			
			if(!beforeNode && spacers) {
				beforeNode = toolbar.childNodes[toolbar.childNodes.length - spacers];
			} else {
				for(let j = 0; j < spacers; j++) {
					beforeNode = beforeNode.previousSibling;
				}
			}
			
			toolbar.insertItem(aWidgetId, beforeNode, null, false);
			toolbar.setAttribute("currentset", buttonSet);
			doc.persist(toolbar.id, "currentset");
			
			return;
		}
	}
}


var windowListener = {
	
	onOpenWindow: function(aWindow) {
		const domWindow = aWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
								 .getInterface(Components.interfaces.nsIDOMWindowInternal || Components.interfaces.nsIDOMWindow);
		
		domWindow.addEventListener("load", function onLoad() {
			domWindow.removeEventListener("load", onLoad, false);
			
			if(gLoadedWidgets) {
				for (let aProperties of gLoadedWidgets) {
					addWidgetToWindow(domWindow, aProperties);
				}
			}
		}, false);
	},
	
	onCloseWindow: function(aWindow) {},
	
	onWindowTitleChange: function(aWindow, aTitle) {}
};

